@@ -56,6 +56,26 @@ struct _finfo {
   struct _finfo* next;
 };
 
+static const char *arbitrary_fields[] = {
+  "Package-Type",
+  "Subarchitecture",
+  "Kernel-Version",
+  "Installer-Menu-Item",
+  "Homepage",
+  "Tag",
+  NULL
+};
+
+static int known_arbitrary_field(const struct arbitraryfield *field) {
+  const char **known;
+
+  for (known= arbitrary_fields; *known; known++)
+    if (strcasecmp(field->name, *known) == 0)
+      return 1;
+
+  return 0;
+}
+
 /* Do a quick check if vstring is a valid versionnumber. Valid in this case
  * means it contains at least one digit. If an error is found increment
  * *errs.
@@ -80,10 +100,10 @@ static struct _finfo* getfi(const char* root, int fd) {
   size_t rl = strlen(root);
 
   if (fn == NULL) {
-    fnlen=rl+2048;
-    fn=(char*)malloc(fnlen);
-  } else if (fnlen < (rl+2048)) {
-    fnlen=rl+2048;
+    fnlen = rl + MAXFILENAME;
+    fn = m_malloc(fnlen);
+  } else if (fnlen < (rl + MAXFILENAME)) {
+    fnlen = rl + MAXFILENAME;
     fn=(char*)realloc(fn,fnlen);
   }
   i=sprintf(fn,"%s/",root);
@@ -91,7 +111,7 @@ static struct _finfo* getfi(const char* root, int fd) {
   while (1) {
     int	res;
     if (i>=fnlen) {
-      fnlen+=2048;
+      fnlen += MAXFILENAME;
       fn=(char*)realloc(fn,fnlen);
     }
     if ((res=read(fd, (fn+i), sizeof(*fn)))<0) {
@@ -106,12 +126,14 @@ static struct _finfo* getfi(const char* root, int fd) {
       break;
 
     i++;
-    assert(i<2048);
+
+    if (i >= MAXFILENAME)
+      ohshit(_("file name '%.50s...' is too long"), fn + rl + 1);
   }
 
-  fi=(struct _finfo*)malloc(sizeof(struct _finfo));
+  fi = m_malloc(sizeof(struct _finfo));
   lstat(fn, &(fi->st));
-  fi->fn=strdup(fn+rl+1);
+  fi->fn = m_strdup(fn + rl + 1);
   fi->next=NULL;
   return fi;
 }
@@ -147,7 +169,7 @@ static void free_filist(struct _finfo* fi) {
  */
 void do_build(const char *const *argv) {
   static const char *const maintainerscripts[]= {
-    PREINSTFILE, POSTINSTFILE, PRERMFILE, POSTRMFILE, 0
+    PREINSTFILE, POSTINSTFILE, PRERMFILE, POSTRMFILE, NULL
   };
   
   char *m;
@@ -171,11 +193,12 @@ void do_build(const char *const *argv) {
   /* template for our tempfiles */
   if ((envbuf= getenv("TMPDIR")) == NULL)
     envbuf= P_tmpdir;
-  tfbuf = (char *)malloc(strlen(envbuf)+13);
+  tfbuf = m_malloc(strlen(envbuf) + 13);
   strcpy(tfbuf,envbuf);
   strcat(tfbuf,"/dpkg.XXXXXX");
   subdir= 0;
-  if ((debar= *argv++) !=0) {
+  debar = *argv++;
+  if (debar != NULL) {
     if (*argv) badusage(_("--build takes at most two arguments"));
     if (debar) {
       if (stat(debar,&debarstab)) {
@@ -221,6 +244,9 @@ void do_build(const char *const *argv) {
       warns++;
     }
     for (field= checkedinfo->available.arbs; field; field= field->next) {
+      if (known_arbitrary_field(field))
+        continue;
+
       fprintf(stderr, _("warning, `%s' contains user-defined field `%s'\n"),
               controlfile, field->name);
       warns++;
@@ -315,14 +341,17 @@ void do_build(const char *const *argv) {
    * build something. Lets start by making the ar-wrapper.
    */
   if (!(ar=fopen(debar,"wb"))) ohshite(_("unable to create `%.255s'"),debar);
-  if (setvbuf(ar, 0, _IONBF, 0)) ohshite(_("unable to unbuffer `%.255s'"),debar);
+  if (setvbuf(ar, NULL, _IONBF, 0))
+    ohshite(_("unable to unbuffer `%.255s'"), debar);
   /* Fork a tar to package the control-section of the package */
+  unsetenv("TAR_OPTIONS");
   m_pipe(p1);
   if (!(c1= m_fork())) {
     m_dup2(p1[1],1); close(p1[0]); close(p1[1]);
     if (chdir(directory)) ohshite(_("failed to chdir to `%.255s'"),directory);
     if (chdir(BUILDCONTROLDIR)) ohshite(_("failed to chdir to .../DEBIAN"));
-    execlp(TAR,"tar","-cf","-",".",(char*)0); ohshite(_("failed to exec tar -cf"));
+    execlp(TAR, "tar", "-cf", "-", "--format=gnu", ".", NULL);
+    ohshite(_("failed to exec tar -cf"));
   }
   close(p1[1]);
   /* Create a temporary file to store the control data in. Immediately unlink
@@ -340,7 +369,7 @@ void do_build(const char *const *argv) {
   /* And run gzip to compress our control archive */
   if (!(c2= m_fork())) {
     m_dup2(p1[0],0); m_dup2(gzfd,1); close(p1[0]); close(gzfd);
-    compress_cat(GZ, 0, 1, "9", _("control"));
+    compress_cat(compress_type_gzip, 0, 1, "9", _("control"));
   }
   close(p1[0]);
   waitsubproc(c2,"gzip -9c",0);
@@ -353,7 +382,7 @@ void do_build(const char *const *argv) {
     if (fprintf(ar, "%-8s\n%ld\n", OLDARCHIVEVERSION, (long)controlstab.st_size) == EOF)
       werr(debar);
   } else {
-    thetime= time(0);
+    thetime = time(NULL);
     if (fprintf(ar,
                 "!<arch>\n"
                 "debian-binary   %-12lu0     0     100644  %-10ld`\n"
@@ -394,7 +423,7 @@ void do_build(const char *const *argv) {
     m_dup2(p1[0],0); close(p1[0]); close(p1[1]);
     m_dup2(p2[1],1); close(p2[0]); close(p2[1]);
     if (chdir(directory)) ohshite(_("failed to chdir to `%.255s'"),directory);
-    execlp(TAR,"tar","-cf", "-", "--null", "-T", "-", "--no-recursion", (char*)0);
+    execlp(TAR, "tar", "-cf", "-", "--format=gnu", "--null", "-T", "-", "--no-recursion", NULL);
     ohshite(_("failed to exec tar -cf"));
   }
   close(p1[0]);
@@ -415,7 +444,8 @@ void do_build(const char *const *argv) {
   if (!(c3= m_fork())) {
     m_dup2(p3[1],1); close(p3[0]); close(p3[1]);
     if (chdir(directory)) ohshite(_("failed to chdir to `%.255s'"),directory);
-    execlp(FIND,"find",".","-path","./" BUILDCONTROLDIR,"-prune","-o","-print0",(char*)0);
+    execlp(FIND, "find", ".", "-path", "./" BUILDCONTROLDIR, "-prune", "-o",
+           "-print0", NULL);
     ohshite(_("failed to exec find"));
   }
   close(p3[1]);
@@ -444,11 +474,20 @@ void do_build(const char *const *argv) {
   if (!oldformatflag) {
     const char *datamember;
     switch (compress_type) {
-      case GZ: datamember= DATAMEMBER_GZ; break;
-      case BZ2: datamember= DATAMEMBER_BZ2; break;
-      case CAT: datamember= DATAMEMBER_CAT; break;
-      default:
-        ohshit(_("Internal error, compress_type `%i' unknown!"), compress_type);
+    case compress_type_gzip:
+      datamember = DATAMEMBER_GZ;
+      break;
+    case compress_type_bzip2:
+      datamember = DATAMEMBER_BZ2;
+      break;
+    case compress_type_lzma:
+      datamember = DATAMEMBER_LZMA;
+      break;
+    case compress_type_cat:
+      datamember = DATAMEMBER_CAT;
+      break;
+    default:
+      ohshit(_("Internal error, compress_type `%i' unknown!"), compress_type);
     }
     if (fstat(gzfd,&datastab)) ohshite("_(failed to fstat tmpfile (data))");
     if (fprintf(ar,
