@@ -29,6 +29,7 @@ extern "C" {
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -36,7 +37,7 @@ extern "C" {
 #include <ctype.h>
 #include <assert.h>
 
-#include <term.h>
+#include <ncursesw/term.h>
 
 extern "C" {
 #include <dpkg.h>
@@ -47,10 +48,6 @@ extern "C" {
 #include "bindings.h"
 #include "pkglist.h"
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
 const char thisname[]= DSELECT;
 const char printforhelp[]= N_("Type dselect --help for help.");
 
@@ -158,14 +155,18 @@ static const char licensestring[]= N_(
       "later for copying conditions. There is NO warranty.\n"
       "See %s --license for copyright and license details.\n");
 
-static void printversion(void) {
+void
+printversion(void)
+{
   if (printf(gettext(programdesc), DSELECT, DPKG_VERSION_ARCH) < 0)
     werr("stdout");
   if (printf(gettext(copyrightstring)) < 0) werr("stdout");
   if (printf(gettext(licensestring), DSELECT) < 0) werr("stdout");
 }
 
-static void usage(void) {
+void
+usage(void)
+{
   int i;
   if (printf(_(
 "Usage: %s [<option> ...] [<action> ...]\n"
@@ -210,15 +211,6 @@ static void usage(void) {
 /* These are called by C code, so need to have C calling convention */
 extern "C" {
 
-  static void helponly(const struct cmdinfo*, const char*) NONRETURNING;
-  static void helponly(const struct cmdinfo*, const char*) {
-    usage(); exit(0);
-  }
-  static void versiononly(const struct cmdinfo*, const char*) NONRETURNING;
-  static void versiononly(const struct cmdinfo*, const char*) {
-    printversion(); exit(0);
-  }
-
   static void setdebug(const struct cmdinfo*, const char *v) {
     debug= fopen(v,"a");
     if (!debug) ohshite(_("couldn't open debug file `%.255s'\n"),v);
