@@ -2,7 +2,7 @@
  * libdpkg - Debian packaging suite library routines
  * compat.c - compatibility functions
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  * License along with dpkg; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 
 #include <errno.h>
 #include <signal.h>
@@ -30,7 +31,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include <config.h>
 #include <dpkg.h>
 
 #ifndef HAVE_VSNPRINTF
@@ -101,7 +101,7 @@ const char *strsignal(int e) {
 }
 #endif
 
-#ifndef SYS_SIGLIST_DECLARED
+#ifndef HAVE_DECL_SYS_SIGLIST
 const char *const sys_siglist[32] = {
   "SIGHUP",      /*1*/
   "SIGINT",      /*2*/
