@@ -72,6 +72,9 @@ void blankpackage(struct pkginfo *pigp) {
   pigp->available.valid= 0;
   pigp->clientdata= NULL;
   pigp->color= white;
+  pigp->trigaw.head = pigp->trigaw.tail = NULL;
+  pigp->othertrigaw_head = NULL;
+  pigp->trigpend_head = NULL;
   blankpackageperfile(&pigp->installed);
   blankpackageperfile(&pigp->available);
 }
@@ -118,10 +121,8 @@ int informative(struct pkginfo *pkg, struct pkginfoperfile *info) {
 
 struct pkginfo *findpackage(const char *inname) {
   struct pkginfo **pointerp, *newpkg;
-  char *name = strdup(inname), *p;
+  char *name = m_strdup(inname), *p;
 
-  if (name == NULL)
-    ohshite(_("couldn't allocate memory for strdup in findpackage(%s)"),inname);
   p= name;
   while(*p) { *p= tolower(*p); p++; }
   
