@@ -14,6 +14,9 @@
 #include <grp.h>
 #include <errno.h>
 #include <tarfn.h>
+#include <dpkg.h>
+
+#include "strnlen.h"
 
 struct TarHeader {
 	char Name[100];
@@ -33,8 +36,7 @@ struct TarHeader {
 };
 typedef struct TarHeader	TarHeader;
 
-static const unsigned int	TarChecksumOffset
-	= (unsigned int)&(((TarHeader *)NULL)->Checksum);
+static const size_t TarChecksumOffset = offsetof(TarHeader, Checksum);
 
 /* Octal-ASCII-to-long */
 static long
@@ -54,14 +56,14 @@ OtoL(const char * s, int size)
 }
 
 /* String block to C null-terminated string */
-char *
+static char *
 StoC(const char *s, int size)
 {
 	int	len;
 	char *	str;
 
 	len = strnlen(s, size);
-	str = malloc(len + 1);
+	str = m_malloc(len + 1);
 	memcpy(str, s, len);
 	str[len] = 0;
 
@@ -119,7 +121,7 @@ typedef struct symlinkList {
 	struct symlinkList *next;
 } symlinkList;
 
-extern int
+int
 TarExtractor(
  void *			userData
 ,const TarFunctions *	functions)
@@ -137,9 +139,11 @@ TarExtractor(
        next_long_name = NULL;
        next_long_link = NULL;
        long_read = 0;
-	symListBottom = symListPointer = symListTop = malloc(sizeof(symlinkList));
+	symListBottom = symListPointer = symListTop = m_malloc(sizeof(symlinkList));
 	symListTop->next = NULL;
 
+	h.Name = NULL;
+	h.LinkName = NULL;
 	h.UserData = userData;
 
 	while ( (status = functions->Read(userData, buffer, 512)) == 512 ) {
@@ -195,24 +199,10 @@ TarExtractor(
 			break;
 		case SymbolicLink:
 			memcpy(&symListBottom->h, &h, sizeof(TarInfo));
-			if ((symListBottom->h.Name = strdup(h.Name)) == NULL) {
-				status = -1;
-				errno = 0;
-				break;
-			}
-			if ((symListBottom->h.LinkName = strdup(h.LinkName)) == NULL) {
-				free(symListBottom->h.Name);
-				status = -1;
-				errno = 0;
-				break;
-			}
-			if ((symListBottom->next = malloc(sizeof(symlinkList))) == NULL) {
-				free(symListBottom->h.LinkName);
-				free(symListBottom->h.Name);
-				status = -1;
-				errno = 0;
-				break;
-			}
+			symListBottom->h.Name = m_strdup(h.Name);
+			symListBottom->h.LinkName = m_strdup(h.LinkName);
+			symListBottom->next = m_malloc(sizeof(symlinkList));
+
 			symListBottom = symListBottom->next;
 			symListBottom->next = NULL;
 			status = 0;
@@ -233,12 +223,7 @@ TarExtractor(
                  if (*longp)
                    free(*longp);
 
-                 if (NULL == (*longp = (char *)malloc(h.Size))) {
-                   /* malloc failed, so bail */
-                   errno = 0;
-		   status = -1;
-		   break;
-                 }
+		*longp = m_malloc(h.Size);
                  bp = *longp;
 
                  // the way the GNU long{link,name} stuff works is like this:  
