@@ -53,8 +53,9 @@ static security_context_t scontext    = NULL;
 #include "main.h"
 #include "archives.h"
 
-/* We shouldn't need anymore than 10 conflictors */
-struct pkginfo *conflictor[20];
+#define MAXCONFLICTORS 20
+
+struct pkginfo *conflictor[MAXCONFLICTORS];
 int cflict_index = 0;
 
 /* snprintf(3) doesn't work if format contains %.<nnn>s and an argument has
@@ -80,39 +81,39 @@ int cflict_index = 0;
 static char *
 quote_filename(char *buf, int size, char *s)
 {
-       char *r = buf;
-       while (size > 0) {
-               switch (*s) {
-               case '\0':
-                       *buf = '\0';
-                       return r;
-               case '\\':
-                       *buf++ = '\\';
-                       *buf++ = '\\';
-                       size -= 2;
-                       break;
-               default:
-                       if (((*s)&0x80) == 0) {
-                               *buf++ = *s++;
-                               --size;
-                       } else {
-                               if (size > 4) {
-                                       sprintf(buf, "\\%03o",
-                                               *(unsigned char *)s);
-                                       size -= 4;
-                                       buf += 4;
-                                       s++;
-                               } else {
-                                       /* buffer full */
-                                       *buf = '\0'; /* XXX */
-                                       return r;
-                               }
-                       }
-               }
-       }
-       *buf = '\0'; /* XXX */
-       return r;
+  char *r = buf;
 
+  while (size > 0) {
+    switch (*s) {
+    case '\0':
+      *buf = '\0';
+      return r;
+    case '\\':
+      *buf++ = '\\';
+      *buf++ = '\\';
+      size -= 2;
+      break;
+    default:
+      if (((*s) & 0x80) == 0) {
+        *buf++ = *s++;
+        --size;
+      } else {
+        if (size > 4) {
+          sprintf(buf, "\\%03o", *(unsigned char *)s);
+          size -= 4;
+          buf += 4;
+          s++;
+        } else {
+          /* buffer full */
+          *buf = '\0'; /* XXX */
+          return r;
+        }
+      }
+    }
+  }
+  *buf = '\0'; /* XXX */
+
+  return r;
 }
 
 /* special routine to handle partial reads from the tarfile */
@@ -148,7 +149,7 @@ static void ensureobstackinit(void) {
 /* destroy the obstack */
 static void destroyobstack(void) {
   if (tarobs_init) {
-    obstack_free(&tar_obs, 0);
+    obstack_free(&tar_obs, NULL);
     tarobs_init= 0;
   }
 }
@@ -233,7 +234,7 @@ int fnameidlu;
 struct varbuf fnamevb;
 struct varbuf fnametmpvb;
 struct varbuf fnamenewvb;
-struct packageinlist *deconfigure= 0;
+struct packageinlist *deconfigure = NULL;
 
 static time_t currenttime;
 
@@ -321,7 +322,9 @@ struct fileinlist *addfiletolist(struct tarcontext *tc,
   
   nifd= obstack_alloc(&tar_obs, sizeof(struct fileinlist));
   nifd->namenode= namenode;
-  nifd->next= 0; *tc->newfilesp= nifd; tc->newfilesp= &nifd->next;
+  nifd->next = NULL;
+  *tc->newfilesp = nifd;
+  tc->newfilesp = &nifd->next;
   return nifd;
 }
 
@@ -335,7 +338,8 @@ static int linktosameexistingdir(const struct TarInfo *ti,
   statr= stat(fname, &oldstab);
   if (statr) {
     if (!(errno == ENOENT || errno == ELOOP || errno == ENOTDIR))
-      ohshite("failed to stat (dereference) existing symlink `%.250s'", fname);
+      ohshite(_("failed to stat (dereference) existing symlink `%.250s'"),
+              fname);
     return 0;
   }
   if (!S_ISDIR(oldstab.st_mode)) return 0;
@@ -355,8 +359,8 @@ static int linktosameexistingdir(const struct TarInfo *ti,
   statr= stat(symlinkfn->buf, &newstab);
   if (statr) {
     if (!(errno == ENOENT || errno == ELOOP || errno == ENOTDIR))
-      ohshite("failed to stat (dereference) proposed new symlink target"
-              " `%.250s' for symlink `%.250s'", symlinkfn->buf, fname);
+      ohshite(_("failed to stat (dereference) proposed new symlink target"
+                " `%.250s' for symlink `%.250s'"), symlinkfn->buf, fname);
     return 0;
   }
   if (!S_ISDIR(newstab.st_mode)) return 0;
@@ -367,12 +371,13 @@ static int linktosameexistingdir(const struct TarInfo *ti,
 
 int tarobject(struct TarInfo *ti) {
   static struct varbuf conffderefn, hardlinkfn, symlinkfn;
+  static int fd;
   const char *usename;
 
   struct conffile *conff;
   struct tarcontext *tc= (struct tarcontext*)ti->UserData;
-  int statr, fd, i, existingdirectory, keepexisting;
-  size_t r;
+  int statr, i, existingdirectory, keepexisting;
+  ssize_t r;
   struct stat stab, stabtmp;
   char databuf[TARBLKSZ];
   struct fileinlist *nifd, **oldnifd;
@@ -403,14 +408,19 @@ int tarobject(struct TarInfo *ti) {
 
   if (nifd->namenode->divert && nifd->namenode->divert->camefrom) {
     divpkg= nifd->namenode->divert->pkg;
-    forcibleerr(fc_overwritediverted,
-                _("trying to overwrite `%.250s', which is the "
-                "diverted version of `%.250s'%.10s%.100s%.10s"),
-                nifd->namenode->name,
-                nifd->namenode->divert->camefrom->name,
-                divpkg ? _(" (package: ") : "",
-                divpkg ? divpkg->name : "",
-                divpkg ? ")" : "");
+
+    if (divpkg) {
+      forcibleerr(fc_overwritediverted,
+                  _("trying to overwrite `%.250s', which is the "
+                    "diverted version of `%.250s' (package: %.100s)"),
+                  nifd->namenode->name, nifd->namenode->divert->camefrom->name,
+                  divpkg->name);
+    } else {
+      forcibleerr(fc_overwritediverted,
+                  _("trying to overwrite `%.250s', which is the "
+                    "diverted version of `%.250s'"),
+                  nifd->namenode->name, nifd->namenode->divert->camefrom->name);
+    }
   }
 
   usename= namenodetouse(nifd->namenode,tc->pkg)->name + 1; /* Skip the leading `/' */
@@ -532,10 +542,11 @@ int tarobject(struct TarInfo *ti) {
 		stabtmp.st_ino == stab.st_ino)
 	      break;
 	  }
-	  if (conff)
+	  if (conff) {
 	    debug(dbg_eachfiledetail,"tarobject other's obsolete conffile");
 	    /* processarc.c will have copied its hash already. */
 	    continue;
+	  }
 	}
 
         if (does_replace(tc->pkg,&tc->pkg->available,otherpkg)) {
@@ -566,17 +577,11 @@ int tarobject(struct TarInfo *ti) {
     }
   }
        
-  /* Now, at this stage we want to make sure neither of .dpkg-new and .dpkg-tmp
-   * are hanging around.
-   */
-  ensure_pathname_nonexisting(fnamenewvb.buf);
-  ensure_pathname_nonexisting(fnametmpvb.buf);
-
   if (existingdirectory) return 0;
   if (keepexisting) {
     obstack_free(&tar_obs, nifd);
     tc->newfilesp= oldnifd;
-    *oldnifd = 0;
+    *oldnifd = NULL;
 
     /* We need to advance the tar file to the next object, so read the
      * file data and set it to oblivion.
@@ -591,11 +596,17 @@ int tarobject(struct TarInfo *ti) {
     return 0;
   }
 
+  /* Now, at this stage we want to make sure neither of .dpkg-new and .dpkg-tmp
+   * are hanging around.
+   */
+  ensure_pathname_nonexisting(fnamenewvb.buf);
+  ensure_pathname_nonexisting(fnametmpvb.buf);
+
   /* Now we start to do things that we need to be able to undo
    * if something goes wrong.  Watch out for the CLEANUP comments to
    * keep an eye on what's installed on the disk at each point.
    */
-  push_cleanup(cu_installnew,~ehflag_normaltidy, 0,0, 1,(void*)nifd);
+  push_cleanup(cu_installnew, ~ehflag_normaltidy, NULL, 0, 1, (void *)nifd);
 
   /* CLEANUP: Now we either have the old file on the disk, or not, in
    * its original filename.
@@ -641,7 +652,7 @@ int tarobject(struct TarInfo *ti) {
      */
     fd= open(fnamenewvb.buf, (O_CREAT|O_EXCL|O_WRONLY), 0);
     if (fd < 0) ohshite(_("unable to create `%.255s'"),ti->Name);
-    push_cleanup(cu_closefd,ehflag_bombout, 0,0, 1,&fd);
+    push_cleanup(cu_closefd, ehflag_bombout, NULL, 0, 1, &fd);
     debug(dbg_eachfiledetail,"tarobject NormalFile[01] open size=%lu",
           (unsigned long)ti->Size);
     { char fnamebuf[256];
@@ -758,7 +769,7 @@ int tarobject(struct TarInfo *ti) {
         varbufextend(&symlinkfn);
         r= readlink(fnamevb.buf,symlinkfn.buf,symlinkfn.size);
         if (r<0) ohshite(_("unable to read link `%.255s'"),ti->Name);
-      } while (r == symlinkfn.size);
+      } while ((size_t)r == symlinkfn.size);
       symlinkfn.used= r; varbufaddc(&symlinkfn,0);
       if (symlink(symlinkfn.buf,fnametmpvb.buf))
         ohshite(_("unable to make backup symlink for `%.255s'"),ti->Name);
@@ -791,6 +802,7 @@ int tarobject(struct TarInfo *ti) {
        if(setfscreatecon(scontext) < 0)
          perror("Error setting security context for next file object:");
        freecon(scontext);
+       scontext = NULL;
      }
         
 #endif /* WITH_SELINUX */
@@ -822,42 +834,124 @@ int tarobject(struct TarInfo *ti) {
   return 0;
 }
 
-static int try_remove_can(struct deppossi *pdep,
-                          struct pkginfo *fixbyrm,
+static int try_deconfigure_can(int (*force_p)(struct deppossi*),
+                               struct pkginfo *pkg,
+                               struct deppossi *pdep,
+                               const char *action,
+                               struct pkginfo *removal,
                           const char *why) {
+  /* Also checks whether the pdep is forced, first, according to force_p.
+   * force_p may be 0 in which case nothing is considered forced.
+   *
+   * Action is a string describing the action which causes the
+   * deconfiguration:
+   *     removal of <package>         (due to Conflicts+Depends   removal!=0)
+   *     installation of <package>    (due to Breaks              removal==0)
+   *
+   * Return values:  2: forced (no deconfiguration needed, why is printed)
+   *                 1: deconfiguration queued ok (no message printed)
+   *                 0: not possible (why is printed)
+   */
   struct packageinlist *newdeconf;
   
-  if (force_depends(pdep)) {
+  if (force_p && force_p(pdep)) {
     fprintf(stderr, _("dpkg: warning - "
-            "ignoring dependency problem with removal of %s:\n%s"),
-            fixbyrm->name, why);
-    return 1;
+                      "ignoring dependency problem with %s:\n%s"),
+            action, why);
+    return 2;
   } else if (f_autodeconf) {
-    if (pdep->up->up->installed.essential) {
+    if (pkg->installed.essential) {
       if (fc_removeessential) {
         fprintf(stderr, _("dpkg: warning - considering deconfiguration of essential\n"
-                " package %s, to enable removal of %s.\n"),
-                pdep->up->up->name,fixbyrm->name);
+                          " package %s, to enable %s.\n"),
+                pkg->name, action);
       } else {
         fprintf(stderr, _("dpkg: no, %s is essential, will not deconfigure\n"
-                " it in order to enable removal of %s.\n"),
-                pdep->up->up->name,fixbyrm->name);
+                          " it in order to enable %s.\n"),
+                pkg->name, action);
         return 0;
       }
     }
-    pdep->up->up->clientdata->istobe= itb_deconfigure;
-    newdeconf= malloc(sizeof(struct packageinlist));
+    pkg->clientdata->istobe= itb_deconfigure;
+    newdeconf = m_malloc(sizeof(struct packageinlist));
     newdeconf->next= deconfigure;
-    newdeconf->pkg= pdep->up->up;
+    newdeconf->pkg= pkg;
+    newdeconf->xinfo= removal;
     deconfigure= newdeconf;
     return 1;
   } else {
-    fprintf(stderr, _("dpkg: no, cannot remove %s (--auto-deconfigure will help):\n%s"),
-            fixbyrm->name, why);
+    fprintf(stderr, _("dpkg: no, cannot proceed with %s (--auto-deconfigure will help):\n%s"),
+            action, why);
     return 0;
   }
 }
 
+static int try_remove_can(struct deppossi *pdep,
+                          struct pkginfo *fixbyrm,
+                          const char *why) {
+  char action[512];
+  sprintf(action, _("removal of %.250s"), fixbyrm->name);
+  return try_deconfigure_can(force_depends, pdep->up->up, pdep,
+                             action, fixbyrm, why);
+}
+
+void check_breaks(struct dependency *dep, struct pkginfo *pkg,
+                  const char *pfilename) {
+  struct pkginfo *fixbydeconf;
+  struct varbuf why;
+  int ok;
+
+  varbufinit(&why);
+
+  fixbydeconf = NULL;
+  if (depisok(dep, &why, &fixbydeconf, 0)) {
+    varbuffree(&why);
+    return;
+  }
+
+  varbufaddc(&why, 0);
+
+  if (fixbydeconf && f_autodeconf) {
+    char action[512];
+
+    ensure_package_clientdata(fixbydeconf);
+    assert(fixbydeconf->clientdata->istobe == itb_normal);
+
+    sprintf(action, _("installation of %.250s"), pkg->name);
+    fprintf(stderr, _("dpkg: considering deconfiguration of %s,"
+                      " which would be broken by %s ...\n"),
+            fixbydeconf->name, action);
+
+    ok= try_deconfigure_can(force_breaks, fixbydeconf, dep->list,
+                            action, NULL, why.buf);
+    if (ok == 1) {
+      fprintf(stderr, _("dpkg: yes, will deconfigure %s (broken by %s).\n"),
+              fixbydeconf->name, pkg->name);
+    }
+  } else {
+    fprintf(stderr, _("dpkg: regarding %s containing %s:\n%s"),
+            pfilename, pkg->name, why.buf);
+    ok= 0;
+  }
+  varbuffree(&why);
+  if (ok > 0) return;
+
+  if (force_breaks(dep->list)) {
+    fprintf(stderr, _("dpkg: warning - ignoring breakage,"
+                      " may proceed anyway !\n"));
+    return;
+  }
+
+  if (fixbydeconf && !f_autodeconf) {
+    ohshit(_("installing %.250s would break %.250s, and\n"
+             " deconfiguration is not permitted (--auto-deconfigure might help)"),
+           pkg->name, fixbydeconf->name);
+  } else {
+    ohshit(_("installing %.250s would break existing software"),
+           pkg->name);
+  }
+}
+
 void check_conflict(struct dependency *dep, struct pkginfo *pkg,
                     const char *pfilename) {
   struct pkginfo *fixbyrm;
@@ -868,7 +962,7 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
   varbufinit(&conflictwhy);
   varbufinit(&removalwhy);
 
-  fixbyrm= 0;
+  fixbyrm = NULL;
   if (depisok(dep, &conflictwhy, &fixbyrm, 0)) {
     varbuffree(&conflictwhy);
     varbuffree(&removalwhy);
@@ -888,18 +982,21 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
       fixbyrm->clientdata->istobe= itb_remove;
       fprintf(stderr, _("dpkg: considering removing %s in favour of %s ...\n"),
               fixbyrm->name, pkg->name);
-      if (fixbyrm->status != stat_installed) {
+      if (!(fixbyrm->status == stat_installed ||
+            fixbyrm->status == stat_triggerspending ||
+            fixbyrm->status == stat_triggersawaited)) {
         fprintf(stderr,
                 _("%s is not properly installed - ignoring any dependencies on it.\n"),
                 fixbyrm->name);
-        pdep= 0;
+        pdep = NULL;
       } else {
         for (pdep= fixbyrm->installed.depended;
              pdep;
              pdep= pdep->nextrev) {
           if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends)
             continue;
-          if (depisok(pdep->up, &removalwhy, 0,0)) continue;
+          if (depisok(pdep->up, &removalwhy, NULL, 0))
+            continue;
           varbufaddc(&removalwhy,0);
           if (!try_remove_can(pdep,fixbyrm,removalwhy.buf))
             break;
@@ -915,7 +1012,8 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
                  pdep= pdep->nextrev) {
               if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends)
                 continue;
-              if (depisok(pdep->up, &removalwhy, 0,0)) continue;
+              if (depisok(pdep->up, &removalwhy, NULL, 0))
+                continue;
               varbufaddc(&removalwhy,0);
               fprintf(stderr, _("dpkg"
                       ": may have trouble removing %s, as it provides %s ...\n"),
@@ -941,10 +1039,10 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
         }
       }
       if (!pdep) {
-	/* if this gets triggered, it means a package has > 10 conflicts/replaces
-	 * pairs, which is the package's fault
-	 */
-	assert(cflict_index < (int)sizeof(conflictor));
+	if (cflict_index >= MAXCONFLICTORS)
+	  ohshit(_("package %s has too many Conflicts/Replaces pairs"),
+		 pkg->name);
+
         /* This conflict is OK - we'll remove the conflictor. */
 	conflictor[cflict_index++]= fixbyrm;
         varbuffree(&conflictwhy); varbuffree(&removalwhy);
@@ -987,6 +1085,8 @@ void archivefiles(const char *const *argv) {
   const char **arglist;
   char *p;
 
+  trigproc_install_hooks();
+
   modstatdb_init(admindir,
                  f_noact ?                     msdbrw_readonly
                : cipaction->arg == act_avail ? msdbrw_write
@@ -994,6 +1094,7 @@ void archivefiles(const char *const *argv) {
                :                               msdbrw_needsuperuser);
 
   checkpath();
+  log_message("startup archives %s", cipaction->olong);
   
   if (f_recursive) {
     
@@ -1003,31 +1104,33 @@ void archivefiles(const char *const *argv) {
     m_pipe(pi);
     if (!(fc= m_fork())) {
       const char *const *ap;
-      char **narglist;
       int i;
       m_dup2(pi[1],1); close(pi[0]); close(pi[1]);
       for (i=0, ap=argv; *ap; ap++, i++);
-      narglist= m_malloc(sizeof(char*)*(i+15));
-      narglist[0]= strdup(FIND);
+      arglist = m_malloc(sizeof(char *) * (i + 15));
+      arglist[0] = FIND;
       for (i=1, ap=argv; *ap; ap++, i++) {
         if (strchr(FIND_EXPRSTARTCHARS,(*ap)[0])) {
           char *a;
           a= m_malloc(strlen(*ap)+10);
           strcpy(a,"./");
           strcat(a,*ap);
-          narglist[i]= a;
+          arglist[i] = a;
         } else {
-          narglist[i]= strdup(*ap);
+          arglist[i] = (const char *)*ap;
         }
       }
-      narglist[i++]= strdup("-follow"); /*  When editing these, make sure that     */
-      narglist[i++]= strdup("-name");   /*  arglist is mallocd big enough, above.  */
-      narglist[i++]= strdup(ARCHIVE_FILENAME_PATTERN);
-      narglist[i++]= strdup("-type");
-      narglist[i++]= strdup("f");
-      narglist[i++]= strdup("-print0");
-      narglist[i++]= 0;
-      execvp(FIND, narglist);
+      /* When editing these, make sure that arglist is malloced big enough,
+       * above.
+       */
+      arglist[i++] = "-follow";
+      arglist[i++] = "-name";
+      arglist[i++] = ARCHIVE_FILENAME_PATTERN;
+      arglist[i++] = "-type";
+      arglist[i++] = "f";
+      arglist[i++] = "-print0";
+      arglist[i++] = NULL;
+      execvp(FIND, (char *const *)arglist);
       ohshite(_("failed to exec find for --recursive"));
     }
     close(pi[1]);
@@ -1057,7 +1160,7 @@ void archivefiles(const char *const *argv) {
       arglist[i++]= p;
       while ((c= *p++) != 0);
     }
-    arglist[i]= 0;
+    arglist[i] = NULL;
     argp= arglist;
 
   } else {
@@ -1068,7 +1171,7 @@ void archivefiles(const char *const *argv) {
     
   }
 
-  currenttime= time(0);
+  currenttime = time(NULL);
 
   varbufinit(&fnamevb);
   varbufinit(&fnametmpvb);
@@ -1082,10 +1185,11 @@ void archivefiles(const char *const *argv) {
   ensure_diversions();
   ensure_statoverrides();
   
-  while ((thisarg= *argp++) != 0) {
+  while ((thisarg = *argp++) != NULL) {
     if (setjmp(ejbuf)) {
       error_unwind(ehflag_bombout);
-      if (onerr_abort > 0) break;
+      if (abort_processing)
+        break;
       continue;
     }
     push_error_handler(&ejbuf,print_error_perpackage,thisarg);
@@ -1094,13 +1198,14 @@ void archivefiles(const char *const *argv) {
     if (ferror(stdout)) werr("stdout");
     if (ferror(stderr)) werr("stderr");
     onerr_abort--;
-    set_error_display(0,0);
+    set_error_display(NULL, NULL);
     error_unwind(ehflag_normaltidy);
   }
 
   switch (cipaction->arg) {
   case act_install:
   case act_configure:
+  case act_triggers:
   case act_remove:
   case act_purge:
     process_queue();
@@ -1111,6 +1216,7 @@ void archivefiles(const char *const *argv) {
     internerr("unknown action");
   }
 
+  trigproc_run_deferred();
   modstatdb_shutdown();
 }
 
@@ -1126,7 +1232,6 @@ int wanttoinstall(struct pkginfo *pkg, const struct versionrevision *ver, int sa
    * also return -1 to mean it doesn't know because it would depend on
    * the version number.
    */
-  enum versiondisplayepochwhen needepochs;
   int r;
 
   if (pkg->want != want_install && pkg->want != want_hold) {
@@ -1142,37 +1247,38 @@ int wanttoinstall(struct pkginfo *pkg, const struct versionrevision *ver, int sa
     }
   }
 
-  if (pkg->status != stat_installed) return 1;
+  if (!(pkg->status == stat_installed ||
+        pkg->status == stat_triggersawaited ||
+        pkg->status == stat_triggerspending))
+    return 1;
   if (!ver) return -1;
 
   r= versioncompare(ver,&pkg->installed.version);
   if (r > 0) {
     return 1;
   } else if (r == 0) {
-    if (f_skipsame && /* same version fully installed ? */
-   pkg->status == stat_installed && !(pkg->eflag &= eflagf_reinstreq)) {
+    /* Same version fully installed. */
+    if (f_skipsame) {
       if (saywhy) fprintf(stderr, _("Version %.250s of %.250s already installed, "
              "skipping.\n"),
-             versiondescribe(&pkg->installed.version,vdew_never),
+             versiondescribe(&pkg->installed.version, vdew_nonambig),
              pkg->name);
       return 0;
     } else {
       return 1;
     }
   } else {
-    needepochs= epochsdiffer(&pkg->available.version,&pkg->installed.version) ?
-      vdew_always : vdew_never;
     if (fc_downgrade) {
       if (saywhy) fprintf(stderr, _("%s - warning: downgrading %.250s "
              "from %.250s to %.250s.\n"), DPKG, pkg->name,
-             versiondescribe(&pkg->installed.version,needepochs),
-             versiondescribe(&pkg->available.version,needepochs));
+             versiondescribe(&pkg->installed.version, vdew_nonambig),
+             versiondescribe(&pkg->available.version, vdew_nonambig));
       return 1;
     } else {
       if (saywhy) fprintf(stderr, _("Will not downgrade %.250s from version %.250s "
              "to %.250s, skipping.\n"), pkg->name,
-             versiondescribe(&pkg->installed.version,needepochs),
-             versiondescribe(&pkg->available.version,needepochs));
+             versiondescribe(&pkg->installed.version, vdew_nonambig),
+             versiondescribe(&pkg->available.version, vdew_nonambig));
       return 0;
     }
   }
@@ -1183,7 +1289,7 @@ struct fileinlist *newconff_append(struct fileinlist ***newconffileslastp_io,
   struct fileinlist *newconff;
 
   newconff= m_malloc(sizeof(struct fileinlist));
-  newconff->next= 0;
+  newconff->next = NULL;
   newconff->namenode= namenode;
   **newconffileslastp_io= newconff;
   *newconffileslastp_io= &newconff->next;
