@@ -55,8 +55,6 @@ static enum conffopt promptconfaction(const char* cfgfile, const char* realold,
 		const char* realnew, int useredited, int distedited,
 		enum conffopt what);
 
-extern struct pipef *status_pipes;
-
 
 void deferred_configure(struct pkginfo *pkg) {
 	/* The algorithm for deciding what to configure first is as follows:
@@ -83,7 +81,7 @@ void deferred_configure(struct pkginfo *pkg) {
 	char *cdr2rest;
 	int ok, r, useredited, distedited;
 	struct conffile *conff;
-	char *currenthash= 0, *newdisthash= 0;
+	char *currenthash = NULL, *newdisthash = NULL;
 	struct stat stab;
 	enum conffopt what;
 	static const char *EMPTY_HASH = "-";
@@ -102,13 +100,24 @@ void deferred_configure(struct pkginfo *pkg) {
 			sincenothing= 0; 
 
 	varbufinit(&aemsgs);
-	ok= dependencies_ok(pkg,0,&aemsgs);
+	ok = dependencies_ok(pkg, NULL, &aemsgs);
 	if (ok == 1) {
 		varbuffree(&aemsgs);
 		pkg->clientdata->istobe= itb_installnew;
 		add_to_queue(pkg);
 		return;
-	} else if (ok == 0) {
+	}
+
+	trigproc_reset_cycle();
+	/* At this point removal from the queue is confirmed.  This
+	 * represents irreversible progress wrt trigger cycles.  Only
+	 * packages in stat_unpacked are automatically added to the
+	 * configuration queue, and during configuration and trigger
+	 * processing new packages can't enter into unpacked.
+	 */
+
+	ok = breakses_ok(pkg, &aemsgs) ? ok : 0;
+	if (ok == 0) {
 		sincenothing= 0;
 		varbufaddc(&aemsgs,0);
 		fprintf(stderr,
@@ -131,7 +140,10 @@ void deferred_configure(struct pkginfo *pkg) {
 					" reinstall it before attempting configuration."));
 
 	printf(_("Setting up %s (%s) ...\n"),pkg->name,
-			versiondescribe(&pkg->installed.version,vdew_never));
+	       versiondescribe(&pkg->installed.version, vdew_nonambig));
+	log_action("configure", pkg);
+
+	trig_activate_packageprocessing(pkg);
 
 	if (f_noact) {
 		pkg->status= stat_installed;
@@ -236,6 +248,7 @@ void deferred_configure(struct pkginfo *pkg) {
 					varbufaddstr(&cdr,DPKGDISTEXT);
 					varbufaddc(&cdr,0);
 					strcpy(cdr2rest,DPKGNEWEXT);
+					trig_file_activate_byname(conff->name, pkg);
 					if (rename(cdr2.buf,cdr.buf))
 						fprintf(stderr,
 								_("dpkg: %s: warning - failed to rename `%.250s' to `%.250s': %s\n"),
@@ -271,6 +284,7 @@ void deferred_configure(struct pkginfo *pkg) {
 					printf(_("Installing new version of config file %s ...\n"),conff->name);
 				case cfo_newconff:
 					strcpy(cdr2rest,DPKGNEWEXT);
+					trig_file_activate_byname(conff->name, pkg);
 					if (rename(cdr2.buf,cdr.buf))
 						ohshite(_("unable to install `%.250s' as `%.250s'"),cdr2.buf,cdr.buf);
 					break;
@@ -295,18 +309,14 @@ void deferred_configure(struct pkginfo *pkg) {
 
 	modstatdb_note(pkg);
 
-	if (maintainer_script_installed(pkg, POSTINSTFILE, "post-installation",
-				"configure",
-				informativeversion(&pkg->configversion)
-				? versiondescribe(&pkg->configversion,
-					vdew_nonambig)
-				: "",
-				(char*)0))
-		putchar('\n');
+	maintainer_script_postinst(pkg, "configure",
+	                           informativeversion(&pkg->configversion) ?
+	                           versiondescribe(&pkg->configversion,
+	                                           vdew_nonambig) : "",
+	                           NULL);
 
-	pkg->status= stat_installed;
 	pkg->eflag= eflagv_ok;
-	modstatdb_note(pkg);
+	post_postinst_tasks(pkg, stat_installed);
 }
 
 
@@ -315,7 +325,7 @@ void deferred_configure(struct pkginfo *pkg) {
  * Returns 0 if everything went ok, -1 otherwise.
  */
 int conffderef(struct pkginfo *pkg, struct varbuf *result, const char *in) {
-	static char*	linkreadbuf	= 0;
+	static char *linkreadbuf = NULL;
 	static int	linkreadbufsize	= 0;
 	struct stat	stab;
 	int		r, need;
@@ -407,16 +417,16 @@ static void md5hash(struct pkginfo *pkg, char **hashbuf, const char *fn) {
 	fd=open(fn,O_RDONLY);
 
 	if (fd>=0) {
-		push_cleanup(cu_closefd,ehflag_bombout, 0,0, 1,&fd);
+		push_cleanup(cu_closefd, ehflag_bombout, NULL, 0, 1, &fd);
 		fd_md5(fd, hashbuf, -1, _("md5hash"));
 		pop_cleanup(ehflag_normaltidy); /* fd= open(cdr.buf) */
 		close(fd);
 	} else if (errno==ENOENT) {
-		*hashbuf= strdup(NONEXISTENTFLAG);
+		*hashbuf = m_strdup(NONEXISTENTFLAG);
 	} else {
 		fprintf(stderr, _("dpkg: %s: warning - unable to open conffile %s for hash: %s\n"),
 				pkg->name, fn, strerror(errno));
-		*hashbuf= strdup("-");
+		*hashbuf = m_strdup("-");
 	}
 }
 
@@ -502,7 +512,7 @@ static void suspend(void) {
 			if (!s || !*s)
 				s=DEFAULTSHELL;
 
-			execlp(s,s,"-i",(char*)0);
+			execlp(s, s, "-i", NULL);
 			ohshite(_("failed to exec shell (%.250s)"),s);
 		}
 
@@ -533,25 +543,9 @@ static enum conffopt promptconfaction(const char* cfgfile, const char* realold,
 	if (!(what&cfof_prompt))
 		return what;
 
-	/* if there is a status pipe, send conffile-prompt there */
-	if (status_pipes) {
-	   static struct varbuf *status= NULL;
-	   struct pipef *pipef= status_pipes;
-	   int r;
-	   if (status == NULL) {
-	      status = nfmalloc(sizeof(struct varbuf));
-	      varbufinit(status);
-	   } else
-	      varbufreset(status);
-	   
-	   r= varbufprintf(status, "status: %s : %s : '%s' '%s' %i %i \n", 
+	statusfd_send("status: %s : %s : '%s' '%s' %i %i ",
 			   cfgfile, "conffile-prompt", 
 			   realold, realnew, useredited, distedited);
-	   while (pipef) {
-	      write(pipef->fd, status->buf, r);
-	      pipef= pipef->next;
-	   }
-	}
 
 	do {
 		/* Flush the terminal's input in case the user
