@@ -108,7 +108,7 @@ static int findbreakcyclerecursive(struct pkginfo *pkg, struct cyclesofarlink *s
   }
   thislink.pkg= pkg;
   thislink.back= sofar;
-  thislink.possi= 0;
+  thislink.possi = NULL;
   for (dep= pkg->installed.depends; dep; dep= dep->next) {
     if (dep->type != dep_depends && dep->type != dep_predepends) continue;
     for (possi= dep->list; possi; possi= possi->next) {
@@ -144,22 +144,45 @@ int findbreakcycle(struct pkginfo *pkg) {
     tpkg->color = white;
   }
 
-  return findbreakcyclerecursive(pkg, 0);
+  return findbreakcyclerecursive(pkg, NULL);
 }
 
 void describedepcon(struct varbuf *addto, struct dependency *dep) {
-  varbufaddstr(addto,dep->up->name);
+  const char *fmt;
+  struct varbuf depstr;
+
   switch (dep->type) {
-  case dep_depends:     varbufaddstr(addto, _(" depends on "));     break;
-  case dep_predepends:  varbufaddstr(addto, _(" pre-depends on ")); break;
-  case dep_recommends:  varbufaddstr(addto, _(" recommends "));     break;
-  case dep_breaks:      varbufaddstr(addto, _(" breaks "));         break;
-  case dep_conflicts:   varbufaddstr(addto, _(" conflicts with ")); break;
-  case dep_suggests:    varbufaddstr(addto, _(" suggests ")); break;
-  case dep_enhances:    varbufaddstr(addto, _(" enhances "));       break;
-  default:              internerr("unknown deptype");
+  case dep_depends:
+    fmt = _("%s depends on %s");
+    break;
+  case dep_predepends:
+    fmt = _("%s pre-depends on %s");
+    break;
+  case dep_recommends:
+    fmt = _("%s recommends %s");
+    break;
+  case dep_suggests:
+    fmt = _("%s suggests %s");
+    break;
+  case dep_breaks:
+    fmt = _("%s breaks %s");
+    break;
+  case dep_conflicts:
+    fmt = _("%s conflicts with %s");
+    break;
+  case dep_enhances:
+    fmt = _("%s enhances %s");
+    break;
+  default:
+    internerr("unknown deptype");
   }
-  varbufdependency(addto, dep);
+
+  varbufinit(&depstr);
+  varbufdependency(&depstr, dep);
+  varbufaddc(&depstr, 0);
+
+  varbufprintf(addto, fmt, dep->up->name, depstr.buf);
+  varbuffree(&depstr);
 }
   
 int depisok(struct dependency *dep, struct varbuf *whynot,
@@ -173,6 +196,10 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
    * before a package is unpacked, when it is sufficient for the package
    * to be unpacked provided that both the unpacked and previously-configured
    * versions are acceptable.
+   * On 0 return (`not OK'), *canfixbyremove refers to a package which
+   * if removed (dep_conflicts) or deconfigured (dep_breaks) will fix
+   * the problem.  Caller may pass 0 for canfixbyremove and need not
+   * initialise *canfixbyremove.
    */
   struct deppossi *possi;
   struct deppossi *provider;
@@ -190,6 +217,9 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
 	 dep->type == dep_recommends || dep->type == dep_suggests ||
 	 dep->type == dep_enhances);
   
+  if (canfixbyremove)
+    *canfixbyremove = NULL;
+
   /* The dependency is always OK if we're trying to remove the depend*ing*
    * package.
    */
@@ -197,9 +227,11 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
   case itb_remove: case itb_deconfigure:
     return 1;
   case itb_normal:
-    /* Only `installed' packages can be make dependency problems */
+    /* Only installed packages can be make dependency problems */
     switch (dep->up->status) {
     case stat_installed:
+    case stat_triggerspending:
+    case stat_triggersawaited:
       break;
     case stat_notinstalled: case stat_configfiles: case stat_halfinstalled:
     case stat_halfconfigured: case stat_unpacked:
@@ -214,16 +246,6 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
     internerr("unknown istobe depending");
   }
 
-  if (dep->type == dep_breaks)
-    /* We don't implement this and we can only be in this state
-     * if either a Breaks-ignorant or a Breaks-supporting dpkg
-     * installed the package.  In both cases it's probably too
-     * late to do anything useful about it now in this version
-     * so we just ignore it and hope.
-     * FIXME: implement Breaks
-     */
-    return 1;
-
   /* Describe the dependency, in case we have to moan about it. */
   varbufreset(whynot);
   varbufaddc(whynot, ' ');
@@ -257,6 +279,7 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
       case itb_normal: case itb_preinstall:
         switch (possi->ed->status) {
         case stat_installed:
+        case stat_triggerspending:
           if (versionsatisfied(&possi->ed->installed,possi)) return 1;
           sprintf(linebuf,_("  %.250s is installed, but is version %.250s.\n"),
                   possi->ed->name,
@@ -271,6 +294,7 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
           break;
         case stat_unpacked:
         case stat_halfconfigured:
+        case stat_triggersawaited:
           if (allowunconfigd) {
             if (!informativeversion(&possi->ed->configversion)) {
               sprintf(linebuf, _("  %.250s is unpacked, but has never been configured.\n"),
@@ -291,6 +315,7 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
               return 1;
             }
           }
+          /* Fall through. */
         default:
           sprintf(linebuf, _("  %.250s is %s.\n"),
                   possi->ed->name, gettext(statusstrings[possi->ed->status]));
@@ -357,12 +382,11 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
       }
     }
 
-    if (canfixbyremove) *canfixbyremove= 0;
     return 0;
 
   } else {
     
-    /* It's a conflict.  There's only one main alternative,
+    /* It's conflicts or breaks.  There's only one main alternative,
      * but we also have to consider Providers.  We return `0' as soon
      * as we find something that matches the conflict, and only describe
      * it then.  If we get to the end without finding anything we return `1'.
@@ -372,9 +396,10 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
     nconflicts= 0;
 
     if (possi->ed != possi->up->up) {
-      /* If the package conflicts with itself it must mean that it conflicts
-       * with other packages which provide the same virtual name.  We therefore
-       * don't look at the real package and go on to the virtual ones.
+      /* If the package conflicts with or breaks itself it must mean
+       * other packages which provide the same virtual name.  We
+       * therefore don't look at the real package and go on to the
+       * virtual ones.
        */
       
       switch (possi->ed->clientdata->istobe) {
@@ -390,13 +415,21 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
         nconflicts++;
         *canfixbyremove= possi->ed;
         break;
-      case itb_normal: case itb_deconfigure: case itb_preinstall:
+      case itb_deconfigure:
+        if (dep->type == dep_breaks) break; /* already deconfiguring this */
+        /* fall through */
+      case itb_normal: case itb_preinstall:
         switch (possi->ed->status) {
         case stat_notinstalled: case stat_configfiles:
           break;
-        default:
+        case stat_halfinstalled: case stat_unpacked:
+        case stat_halfconfigured:
+          if (dep->type == dep_breaks) break; /* no problem */
+        case stat_installed:
+        case stat_triggerspending:
+        case stat_triggersawaited:
           if (!versionsatisfied(&possi->ed->installed, possi)) break;
-          sprintf(linebuf, _("  %.250s (version %.250s) is %s.\n"),
+          sprintf(linebuf, _("  %.250s (version %.250s) is present and %s.\n"),
                   possi->ed->name,
                   versiondescribe(&possi->ed->installed.version,vdew_nonambig),
                   gettext(statusstrings[possi->ed->status]));
@@ -425,7 +458,8 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
                 provider->up->up->name, possi->ed->name);
         varbufaddstr(whynot, linebuf);
         /* We can't remove the one we're about to install: */
-        if (canfixbyremove) *canfixbyremove= 0;
+        if (canfixbyremove)
+          *canfixbyremove = NULL;
         return 0;
       }
 
@@ -447,12 +481,20 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
           continue; 
         case itb_remove:
           continue;
-        case itb_normal: case itb_deconfigure: case itb_preinstall:
+        case itb_deconfigure:
+          if (dep->type == dep_breaks) continue; /* already deconfiguring */
+        case itb_normal: case itb_preinstall:
           switch (provider->up->up->status) {
           case stat_notinstalled: case stat_configfiles:
             continue;
-          default:
-            sprintf(linebuf, _("  %.250s provides %.250s and is %s.\n"),
+          case stat_halfinstalled: case stat_unpacked:
+          case stat_halfconfigured:
+            if (dep->type == dep_breaks) break; /* no problem */
+          case stat_installed:
+          case stat_triggerspending:
+          case stat_triggersawaited:
+            sprintf(linebuf,
+                    _("  %.250s provides %.250s and is present and %s.\n"),
                     provider->up->up->name, possi->ed->name,
                     gettext(statusstrings[provider->up->up->status]));
             varbufaddstr(whynot, linebuf);
@@ -469,7 +511,8 @@ int depisok(struct dependency *dep, struct varbuf *whynot,
     }
 
     if (!nconflicts) return 1;
-    if (nconflicts>1) *canfixbyremove= 0;
+    if (nconflicts > 1)
+      *canfixbyremove = NULL;
     return 0;
 
   } /* if (dependency) {...} else {...} */
