@@ -44,6 +44,12 @@
 #include "main.h"
 #include "archives.h"
 
+struct rename_list {
+  struct rename_list *next;
+  char *src;
+  char *dst;
+};
+
 void process_archive(const char *filename) {
   static const struct TarFunctions tf = {
     tarfileread,
@@ -57,7 +63,7 @@ void process_archive(const char *filename) {
    */
   static int p1[2];
   static char cidirtmpnambuf[L_tmpnam+100];
-  static char *cidirbuf=0, *reasmbuf=0;
+  static char *cidirbuf = NULL, *reasmbuf = NULL;
   static struct fileinlist *newconffiles, *newfileslist;
   static enum pkgstatus oldversionstatus;
   static struct varbuf infofnvb, fnvb, depprobwhy;
@@ -83,6 +89,7 @@ void process_archive(const char *filename) {
   struct dirent *de;
   struct stat stab, oldfs;
   struct packageinlist *deconpil, *deconpiltemp;
+  struct rename_list *rename_head = NULL, *rename_node = NULL;
   
   cleanup_pkg_failed= cleanup_conflictor_failed= 0;
   admindirlen= strlen(admindir);
@@ -110,10 +117,10 @@ void process_archive(const char *filename) {
     }
     if (unlink(reasmbuf) && errno != ENOENT)
       ohshite(_("error ensuring `%.250s' doesn't exist"),reasmbuf);
-    push_cleanup(cu_pathname,~0, 0,0, 1,(void*)reasmbuf);
+    push_cleanup(cu_pathname, ~0, NULL, 0, 1, (void *)reasmbuf);
     c1= m_fork();
     if (!c1) {
-      execlp(SPLITTER, SPLITTER,"-Qao",reasmbuf,filename,(char*)0);
+      execlp(SPLITTER, SPLITTER, "-Qao", reasmbuf, filename, NULL);
       ohshite(_("failed to exec dpkg-split to see if it's part of a multiparter"));
     }
     while ((r= waitpid(c1,&status,0)) == -1 && errno == EINTR);
@@ -180,11 +187,11 @@ void process_archive(const char *filename) {
   assert(*cidir && cidirrest[-1] == '/'); cidirrest[-1]= 0;
   ensure_pathname_nonexisting(cidir); cidirrest[-1]= '/';
   
-  push_cleanup(cu_cidir,~0, 0,0, 2,(void*)cidir,(void*)cidirrest);
+  push_cleanup(cu_cidir, ~0, NULL, 0, 2, (void *)cidir, (void *)cidirrest);
   c1= m_fork();
   if (!c1) {
     cidirrest[-1]= 0;
-    execlp(BACKEND, BACKEND,"--control",filename,cidir,(char*)0);
+    execlp(BACKEND, BACKEND, "--control", filename, cidir, NULL);
     ohshite(_("failed to exec dpkg-deb to extract control information"));
   }
   waitsubproc(c1,BACKEND " --control",0);
@@ -194,8 +201,8 @@ void process_archive(const char *filename) {
           &pkg,NULL,NULL);
   if (!pkg->files) {
     pkg->files= nfmalloc(sizeof(struct filedetails));
-    pkg->files->next= 0;
-    pkg->files->name= pkg->files->msdosname= pkg->files->md5sum= 0;
+    pkg->files->next = NULL;
+    pkg->files->name = pkg->files->msdosname = pkg->files->md5sum = NULL;
   }
   /* Always nfmalloc.  Otherwise, we may overwrite some other field(like md5sum). */
   pkg->files->size= nfmalloc(30);
@@ -223,7 +230,7 @@ void process_archive(const char *filename) {
     deconpiltemp= deconpil->next;
     free(deconpil);
   }
-  deconfigure= 0;
+  deconfigure = NULL;
   clear_istobes();
 
   if (!wanttoinstall(pkg,&pkg->available.version,1)) {
@@ -241,6 +248,10 @@ void process_archive(const char *filename) {
       /* Look for things we conflict with. */
       check_conflict(dsearch, pkg, pfilename);
       break;
+    case dep_breaks:
+      /* Look for things we break. */
+      check_breaks(dsearch, pkg, pfilename);
+      break;
     case dep_provides:
       /* Look for things that conflict with what we provide. */
       if (dsearch->list->ed->installed.valid) {
@@ -252,16 +263,6 @@ void process_archive(const char *filename) {
         }
       }
       break;
-    case dep_breaks:
-      fprintf(stderr, _("dpkg: regarding %s containing %s:\n"
-			" package uses Breaks; not supported in this dpkg\n"),
-	      pfilename, pkg->name);
-      if (!force_depends(dsearch->list))
-	ohshit(_("unsupported dependency problem - not installing %.250s"),
-	       pkg->name);
-      fprintf(stderr, _("dpkg: warning - ignoring Breaks !\n"));
-      /* FIXME: implement Breaks */
-      break;
     case dep_suggests:
     case dep_recommends:
     case dep_depends:
@@ -270,7 +271,7 @@ void process_archive(const char *filename) {
       /* Ignore these here. */
       break;
     case dep_predepends:
-      if (!depisok(dsearch,&depprobwhy,0,1)) {
+      if (!depisok(dsearch, &depprobwhy, NULL, 1)) {
         varbufaddc(&depprobwhy,0);
         fprintf(stderr, _("dpkg: regarding %s containing %s, pre-dependency problem:\n%s"),
                 pfilename, pkg->name, depprobwhy.buf);
@@ -288,13 +289,14 @@ void process_archive(const char *filename) {
   
   ensure_allinstfiles_available();
   filesdbinit();
-  
+  trig_file_interests_ensure();
+
   if (pkg->status != stat_notinstalled && pkg->status != stat_configfiles) {
-    log_action("upgrade", pkg);
     printf(_("Preparing to replace %s %s (using %s) ...\n"),
            pkg->name,
            versiondescribe(&pkg->installed.version,vdew_nonambig),
            pfilename);
+    log_action("upgrade", pkg);
   } else {
     printf(_("Unpacking %s (from %s) ...\n"),pkg->name,pfilename);
     log_action("install", pkg);
@@ -305,15 +307,22 @@ void process_archive(const char *filename) {
     return;
   }
 
-  /* OK, we're going ahead.  First we read the conffiles, and copy the
-   * hashes across.
+  /*
+   * OK, we're going ahead.
    */
-  newconffiles= 0; newconffileslastp= &newconffiles;
-  push_cleanup(cu_fileslist,~0, 0, 0, 0);
+
+  trig_activate_packageprocessing(pkg);
+  strcpy(cidirrest, TRIGGERSCIFILE);
+  trig_parse_ci(cidir, NULL, trig_cicb_statuschange_activate, pkg);
+
+  /* Read the conffiles, and copy the hashes across. */
+  newconffiles = NULL;
+  newconffileslastp = &newconffiles;
+  push_cleanup(cu_fileslist, ~0, NULL, 0, 0);
   strcpy(cidirrest,CONFFILESFILE);
   conff= fopen(cidir,"r");
   if (conff) {
-    push_cleanup(cu_closefile,ehflag_bombout, 0,0, 1,(void*)conff);
+    push_cleanup(cu_closefile, ehflag_bombout, NULL, 0, 1, (void *)conff);
     while (fgets(conffilenamebuf,MAXCONFFILENAME-2,conff)) {
       p= conffilenamebuf + strlen(conffilenamebuf);
       assert(p != conffilenamebuf);
@@ -338,7 +347,7 @@ void process_archive(const char *filename) {
        * having that file.  If several packages are listed as owning
        * the file we pick one at random.
        */
-      searchconff= 0;
+      searchconff = NULL;
       for (packageslump= newconff->namenode->packages;
            packageslump;
            packageslump= packageslump->more) {
@@ -388,15 +397,18 @@ void process_archive(const char *filename) {
   
   oldversionstatus= pkg->status;
 
-  assert(oldversionstatus <= stat_configfiles);
+  assert(oldversionstatus <= stat_installed);
   debug(dbg_general,"process_archive oldversionstatus=%s",
         statusstrings[oldversionstatus]);
   
-  if (oldversionstatus == stat_halfconfigured || oldversionstatus == stat_installed) {
+  if (oldversionstatus == stat_halfconfigured ||
+      oldversionstatus == stat_triggersawaited ||
+      oldversionstatus == stat_triggerspending ||
+      oldversionstatus == stat_installed) {
     pkg->eflag |= eflagf_reinstreq;
     pkg->status= stat_halfconfigured;
     modstatdb_note(pkg);
-    push_cleanup(cu_prermupgrade,~ehflag_normaltidy, 0,0, 1,(void*)pkg);
+    push_cleanup(cu_prermupgrade, ~ehflag_normaltidy, NULL, 0, 1, (void *)pkg);
     maintainer_script_alternative(pkg, PRERMFILE, "pre-removal", cidir, cidirrest,
                                   "upgrade", "failed-upgrade");
     pkg->status= stat_unpacked;
@@ -404,40 +416,60 @@ void process_archive(const char *filename) {
     modstatdb_note(pkg);
   }
 
-  for (i = 0 ; i < cflict_index ; i++) {
-    if (!(conflictor[i]->status == stat_halfconfigured ||
-	  conflictor[i]->status == stat_installed)) continue;
-    for (deconpil= deconfigure; deconpil; deconpil= deconpil->next) {
-      printf(_("De-configuring %s, so that we can remove %s ...\n"),
-             deconpil->pkg->name, conflictor[i]->name);
-      deconpil->pkg->status= stat_halfconfigured;
-      modstatdb_note(deconpil->pkg);
-      /* This means that we *either* go and run postinst abort-deconfigure,
-       * *or* queue the package for later configure processing, depending
-       * on which error cleanup route gets taken.
-       */
-      push_cleanup(cu_prermdeconfigure,~ehflag_normaltidy,
-                   ok_prermdeconfigure,ehflag_normaltidy,
-                   3,(void*)deconpil->pkg,
-		   (void*)conflictor[i],(void*)pkg);
+  for (deconpil= deconfigure; deconpil; deconpil= deconpil->next) {
+    struct pkginfo *removing= deconpil->xinfo;
+
+    if (removing)
+      printf(_("De-configuring %s, to allow removal of %s ...\n"),
+             deconpil->pkg->name, removing->name);
+    else
+      printf(_("De-configuring %s ...\n"), deconpil->pkg->name);
+
+    trig_activate_packageprocessing(deconpil->pkg);
+    deconpil->pkg->status= stat_halfconfigured;
+    modstatdb_note(deconpil->pkg);
+
+    /* This means that we *either* go and run postinst abort-deconfigure,
+     * *or* queue the package for later configure processing, depending
+     * on which error cleanup route gets taken.
+     */
+    push_cleanup(cu_prermdeconfigure, ~ehflag_normaltidy,
+                 ok_prermdeconfigure, ehflag_normaltidy,
+                 3, (void*)deconpil->pkg, (void*)removing, (void*)pkg);
+
+    if (removing) {
       maintainer_script_installed(deconpil->pkg, PRERMFILE, "pre-removal",
                                   "deconfigure", "in-favour", pkg->name,
                                   versiondescribe(&pkg->available.version,
                                                   vdew_nonambig),
-                                  "removing", conflictor[i]->name,
-                                  versiondescribe(&conflictor[i]->installed.version,
+                                  "removing", removing->name,
+                                  versiondescribe(&removing->installed.version,
                                                   vdew_nonambig),
-                                  (char*)0);
+                                  NULL);
+    } else {
+      maintainer_script_installed(deconpil->pkg, PRERMFILE, "pre-removal",
+                                  "deconfigure", "in-favour", pkg->name,
+                                  versiondescribe(&pkg->available.version,
+                                                  vdew_nonambig),
+                                  NULL);
     }
+  }
+
+  for (i = 0 ; i < cflict_index; i++) {
+    if (!(conflictor[i]->status == stat_halfconfigured ||
+          conflictor[i]->status == stat_triggersawaited ||
+          conflictor[i]->status == stat_triggerspending ||
+          conflictor[i]->status == stat_installed)) continue;
+    trig_activate_packageprocessing(conflictor[i]);
     conflictor[i]->status= stat_halfconfigured;
     modstatdb_note(conflictor[i]);
-    push_cleanup(cu_prerminfavour,~ehflag_normaltidy, 0,0,
+    push_cleanup(cu_prerminfavour, ~ehflag_normaltidy, NULL, 0,
                  2,(void*)conflictor[i],(void*)pkg);
     maintainer_script_installed(conflictor[i], PRERMFILE, "pre-removal",
                                 "remove", "in-favour", pkg->name,
                                 versiondescribe(&pkg->available.version,
                                                 vdew_nonambig),
-                                (char*)0);
+                                NULL);
     conflictor[i]->status= stat_halfinstalled;
     modstatdb_note(conflictor[i]);
   }
@@ -448,24 +480,24 @@ void process_archive(const char *filename) {
   pkg->status= stat_halfinstalled;
   modstatdb_note(pkg);
   if (oldversionstatus == stat_notinstalled) {
-    push_cleanup(cu_preinstverynew,~ehflag_normaltidy, 0,0,
+    push_cleanup(cu_preinstverynew, ~ehflag_normaltidy, NULL, 0,
                  3,(void*)pkg,(void*)cidir,(void*)cidirrest);
     maintainer_script_new(pkg->name, PREINSTFILE, "pre-installation", cidir, cidirrest,
-                          "install", (char*)0);
+                          "install", NULL);
   } else if (oldversionstatus == stat_configfiles) {
-    push_cleanup(cu_preinstnew,~ehflag_normaltidy, 0,0,
+    push_cleanup(cu_preinstnew, ~ehflag_normaltidy, NULL, 0,
                  3,(void*)pkg,(void*)cidir,(void*)cidirrest);
     maintainer_script_new(pkg->name, PREINSTFILE, "pre-installation", cidir, cidirrest,
                           "install", versiondescribe(&pkg->installed.version,
                                                      vdew_nonambig),
-                          (char*)0);
+                          NULL);
   } else {
-    push_cleanup(cu_preinstupgrade,~ehflag_normaltidy, 0,0,
+    push_cleanup(cu_preinstupgrade, ~ehflag_normaltidy, NULL, 0,
                  4,(void*)pkg,(void*)cidir,(void*)cidirrest,(void*)&oldversionstatus);
     maintainer_script_new(pkg->name, PREINSTFILE, "pre-installation", cidir, cidirrest,
                           "upgrade", versiondescribe(&pkg->installed.version,
                                                      vdew_nonambig),
-                          (char*)0);
+                          NULL);
     printf(_("Unpacking replacement %.250s ...\n"),pkg->name);
   }
   
@@ -542,20 +574,21 @@ void process_archive(const char *filename) {
    */
 
   m_pipe(p1);
-  push_cleanup(cu_closepipe,ehflag_bombout, 0,0, 1,(void*)&p1[0]);
+  push_cleanup(cu_closepipe, ehflag_bombout, NULL, 0, 1, (void *)&p1[0]);
   c1= m_fork();
   if (!c1) {
     m_dup2(p1[1],1); close(p1[0]); close(p1[1]);
-    execlp(BACKEND, BACKEND, "--fsys-tarfile", filename, (char*)0);
+    execlp(BACKEND, BACKEND, "--fsys-tarfile", filename, NULL);
     ohshite(_("unable to exec dpkg-deb to get filesystem archive"));
   }
   close(p1[1]);
+  p1[1] = -1;
 
-  newfileslist= 0; tc.newfilesp= &newfileslist;
-  push_cleanup(cu_fileslist,~0, 0, 0, 0);
+  newfileslist = NULL;
+  tc.newfilesp = &newfileslist;
+  push_cleanup(cu_fileslist, ~0, NULL, 0, 0);
   tc.pkg= pkg;
   tc.backendpipe= p1[0];
-  push_cleanup(cu_closefd,~ehflag_bombout, 0,0, 1,&tc.backendpipe);
 
   r= TarExtractor((void*)&tc, &tf);
   if (r) {
@@ -565,8 +598,9 @@ void process_archive(const char *filename) {
       ohshit(_("corrupted filesystem tarfile - corrupted package archive"));
     }
   }
-  fd_null_copy(tc.backendpipe,-1,_("dpkg-deb: zap possible trailing zeros"));
-  close(tc.backendpipe);
+  fd_null_copy(p1[0], -1, _("dpkg-deb: zap possible trailing zeros"));
+  close(p1[0]);
+  p1[0] = -1;
   waitsubproc(c1,BACKEND " --fsys-tarfile",PROCPIPE);
 
   if (oldversionstatus == stat_halfinstalled || oldversionstatus == stat_unpacked) {
@@ -575,7 +609,7 @@ void process_archive(const char *filename) {
      */
     pkg->status= stat_halfinstalled;
     modstatdb_note(pkg);
-    push_cleanup(cu_postrmupgrade,~ehflag_normaltidy, 0,0, 1,(void*)pkg);
+    push_cleanup(cu_postrmupgrade, ~ehflag_normaltidy, NULL, 0, 1, (void *)pkg);
     maintainer_script_alternative(pkg, POSTRMFILE, "post-removal", cidir, cidirrest,
                                   "upgrade", "failed-upgrade");
   }
@@ -606,14 +640,16 @@ void process_archive(const char *filename) {
     if ((namenode->flags & fnnf_new_conff) ||
         (namenode->flags & fnnf_new_inarchive))
       continue;
+
+    fnamevb.used= fnameidlu;
+    varbufaddstr(&fnamevb, namenodetouse(namenode,pkg)->name);
+    varbufaddc(&fnamevb,0);
+
     if (!stat(namenode->name,&stab) && S_ISDIR(stab.st_mode)) {
       debug(dbg_eachfiledetail, "process_archive: %s is a directory",
 	    namenode->name);
       if (isdirectoryinuse(namenode,pkg)) continue;
     }
-    fnamevb.used= fnameidlu;
-    varbufaddstr(&fnamevb, namenodetouse(namenode,pkg)->name);
-    varbufaddc(&fnamevb,0);
 
     if (lstat(fnamevb.buf, &oldfs)) {
       if (!(errno == ENOENT || errno == ELOOP || errno == ENOTDIR))
@@ -648,7 +684,7 @@ void process_archive(const char *filename) {
        * the process a little leaner. We are only worried about new ones
        * since ones that stayed the same don't really apply here.
        */
-      struct fileinlist *sameas= 0;
+      struct fileinlist *sameas = NULL;
       /* If we can't stat the old or new file, or it's a directory,
        * we leave it up to the normal code
        */
@@ -663,7 +699,7 @@ void process_archive(const char *filename) {
 	      ohshite(_("unable to stat other new file `%.250s'"),
 		      cfile->namenode->name);
 	    memset(cfile->namenode->filestat, 0,
-		   sizeof(cfile->namenode->filestat));
+		   sizeof(*cfile->namenode->filestat));
 	    continue;
 	  }
 	}
@@ -672,7 +708,7 @@ void process_archive(const char *filename) {
 	    oldfs.st_ino == cfile->namenode->filestat->st_ino) {
 	  if (sameas)
 	    fprintf(stderr, _("dpkg: warning - old file `%.250s' is the same"
-		      " as several new files!  (both `%.250s' and `%.250s')"),
+		    " as several new files!  (both `%.250s' and `%.250s')\n"),
 		    fnamevb.buf,
 		    sameas->namenode->name, cfile->namenode->name);
 	  sameas= cfile;
@@ -725,6 +761,16 @@ void process_archive(const char *filename) {
    */
   write_filelist_except(pkg,newfileslist,0);
 
+  /* Trigger interests may have changed.
+   * Firstly we go through the old list of interests deleting them.
+   * Then we go through the new list adding them.
+   */
+  strcpy(cidirrest, TRIGGERSCIFILE);
+  trig_parse_ci(pkgadminfile(pkg, TRIGGERSCIFILE),
+                trig_cicb_interest_delete, NULL, pkg);
+  trig_parse_ci(cidir, trig_cicb_interest_add, NULL, pkg);
+  trig_file_interests_save();
+
   /* We also install the new maintainer scripts, and any other
    * cruft that may have come along with the package.  First
    * we go through the existing scripts replacing or removing
@@ -739,8 +785,8 @@ void process_archive(const char *filename) {
   varbufaddc(&infofnvb,0);
   dsd= opendir(infofnvb.buf);
   if (!dsd) ohshite(_("cannot read info directory"));
-  push_cleanup(cu_closedir,~0, 0,0, 1,(void*)dsd);
-  while ((de= readdir(dsd)) != 0) {
+  push_cleanup(cu_closedir, ~0, NULL, 0, 1, (void *)dsd);
+  while ((de = readdir(dsd)) != NULL) {
     debug(dbg_veryverbose, "process_archive info file `%s'", de->d_name);
     if (de->d_name[0] == '.') continue; /* ignore dotfiles, including `.' and `..' */
     p= strrchr(de->d_name,'.'); if (!p) continue; /* ignore anything odd */
@@ -757,24 +803,42 @@ void process_archive(const char *filename) {
     varbufaddstr(&infofnvb,de->d_name);
     varbufaddc(&infofnvb,0);
     strcpy(cidirrest,p);
-    if (!rename(cidir,infofnvb.buf)) {
-      debug(dbg_scripts, "process_archive info installed %s as %s",
-            cidir, infofnvb.buf);
-    } else if (errno == ENOENT) {
-      /* Right, no new version. */
-      if (unlink(infofnvb.buf))
-        ohshite(_("unable to remove obsolete info file `%.250s'"),infofnvb.buf);
-      debug(dbg_scripts, "process_archive info unlinked %s",infofnvb.buf);
-    } else {
-      ohshite(_("unable to install (supposed) new info file `%.250s'"),cidir);
-    }
+    /* We keep files to rename in a list as doing the rename immediately
+     * might influence the current readdir(), the just renamed file might
+     * be returned a second time as it's actually a new file from the
+     * point of view of the filesystem. */
+    rename_node = m_malloc(sizeof(*rename_node));
+    rename_node->next = rename_head;
+    rename_node->src = m_strdup(cidir);
+    rename_node->dst = m_strdup(infofnvb.buf);
+    rename_head = rename_node;
   }
   pop_cleanup(ehflag_normaltidy); /* closedir */
+
+  while ((rename_node = rename_head)) {
+    if (!rename(rename_node->src, rename_node->dst)) {
+      debug(dbg_scripts, "process_archive info installed %s as %s",
+            rename_node->src, rename_node->dst);
+    } else if (errno == ENOENT) {
+      /* Right, no new version. */
+      if (unlink(rename_node->dst))
+        ohshite(_("unable to remove obsolete info file `%.250s'"),
+                rename_node->dst);
+      debug(dbg_scripts, "process_archive info unlinked %s", rename_node->dst);
+    } else {
+      ohshite(_("unable to install (supposed) new info file `%.250s'"),
+              rename_node->src);
+    }
+    rename_head = rename_node->next;
+    free(rename_node->src);
+    free(rename_node->dst);
+    free(rename_node);
+  }
   
   *cidirrest= 0; /* the directory itself */
   dsd= opendir(cidir);
   if (!dsd) ohshite(_("unable to open temp control directory"));
-  push_cleanup(cu_closedir,~0, 0,0, 1,(void*)dsd);
+  push_cleanup(cu_closedir, ~0, NULL, 0, 1, (void *)dsd);
   while ((de= readdir(dsd))) {
     if (strchr(de->d_name,'.')) {
       debug(dbg_scripts,"process_archive tmp.ci script/file `%s' contains dot",
@@ -825,17 +889,20 @@ void process_archive(const char *filename) {
    * links (linking our deppossi's into the reverse chains)
    * can be done by copy_dependency_links.
    */
-  newdeplist= 0; newdeplistlastp= &newdeplist;
+  newdeplist = NULL;
+  newdeplistlastp = &newdeplist;
   for (dep= pkg->available.depends; dep; dep= dep->next) {
     newdep= nfmalloc(sizeof(struct dependency));
     newdep->up= pkg;
-    newdep->next= 0;
-    newdep->list= 0; newpossilastp= &newdep->list;
+    newdep->next = NULL;
+    newdep->list = NULL;
+    newpossilastp = &newdep->list;
     for (possi= dep->list; possi; possi= possi->next) {
       newpossi= nfmalloc(sizeof(struct deppossi));
       newpossi->up= newdep;
       newpossi->ed= possi->ed;
-      newpossi->next= 0; newpossi->nextrev= newpossi->backrev= 0;
+      newpossi->next = NULL;
+      newpossi->nextrev = newpossi->backrev = NULL;
       newpossi->verrel= possi->verrel;
       if (possi->verrel != dvr_none)
         newpossi->version= possi->version;
@@ -873,10 +940,11 @@ void process_archive(const char *filename) {
   pkg->installed.bugs = pkg->available.bugs;                                   
 
   /* We have to generate our own conffiles structure. */
-  pkg->installed.conffiles= 0; iconffileslastp= &pkg->installed.conffiles;
+  pkg->installed.conffiles = NULL;
+  iconffileslastp = &pkg->installed.conffiles;
   for (cfile= newconffiles; cfile; cfile= cfile->next) {
     newiconff= nfmalloc(sizeof(struct conffile));
-    newiconff->next= 0;
+    newiconff->next = NULL;
     newiconff->name= nfstrsave(cfile->namenode->name);
     newiconff->hash= nfstrsave(cfile->namenode->oldhash);
     newiconff->obsolete= !!(cfile->namenode->flags & fnnf_obs_conff);
@@ -900,7 +968,7 @@ void process_archive(const char *filename) {
    * with them !).
    */
   it= iterpkgstart();
-  while ((otherpkg= iterpkgnext(it)) != 0) {
+  while ((otherpkg = iterpkgnext(it)) != NULL) {
     ensure_package_clientdata(otherpkg);
     if (otherpkg == pkg ||
         otherpkg->status == stat_notinstalled ||
@@ -930,7 +998,8 @@ void process_archive(const char *filename) {
          pdep= pdep->nextrev) {
       if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends &&
           pdep->up->type != dep_recommends) continue;
-      if (depisok(pdep->up, &depprobwhy, 0,0)) continue;
+      if (depisok(pdep->up, &depprobwhy, NULL, 0))
+        continue;
       varbufaddc(&depprobwhy,0);
       debug(dbg_veryverbose,"process_archive cannot disappear: %s",depprobwhy.buf);
       break;
@@ -947,7 +1016,8 @@ void process_archive(const char *filename) {
           if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends &&
               pdep->up->type != dep_recommends)
             continue;
-          if (depisok(pdep->up, &depprobwhy, 0,0)) continue;
+          if (depisok(pdep->up, &depprobwhy, NULL, 0))
+            continue;
           varbufaddc(&depprobwhy,0);
           debug(dbg_veryverbose,"process_archive cannot disappear (provides %s): %s",
                 providecheck->list->ed->name, depprobwhy.buf);
@@ -966,12 +1036,13 @@ void process_archive(const char *filename) {
      * run maintainer scripts and things, as we can't back out.  But
      * what can we do ?  It has to be run this late.
      */
+    trig_activate_packageprocessing(otherpkg);
     maintainer_script_installed(otherpkg, POSTRMFILE,
                                 "post-removal script (for disappearance)",
                                 "disappear", pkg->name, 
                                 versiondescribe(&pkg->available.version,
                                                 vdew_nonambig),
-                                (char*)0);
+                                NULL);
 
     /* OK, now we delete all the stuff in the `info' directory .. */
     varbufreset(&fnvb);
@@ -980,11 +1051,11 @@ void process_archive(const char *filename) {
     infodirbaseused= fnvb.used;
     varbufaddc(&fnvb,0);
     dsd= opendir(fnvb.buf); if (!dsd) ohshite(_("cannot read info directory"));
-    push_cleanup(cu_closedir,~0, 0,0, 1,(void*)dsd);
+    push_cleanup(cu_closedir, ~0, NULL, 0, 1, (void *)dsd);
 
     debug(dbg_general, "process_archive disappear cleaning info directory");
       
-    while ((de= readdir(dsd)) != 0) {
+    while ((de = readdir(dsd)) != NULL) {
       debug(dbg_veryverbose, "process_archive info file `%s'", de->d_name);
       if (de->d_name[0] == '.') continue;
       p= strrchr(de->d_name,'.'); if (!p) continue;
@@ -1004,15 +1075,15 @@ void process_archive(const char *filename) {
     otherpkg->want= want_purge;
     otherpkg->eflag= eflagv_ok;
 
-    otherpkg->installed.depends= 0;
+    otherpkg->installed.depends = NULL;
     otherpkg->installed.essential= 0;
-    otherpkg->installed.description= otherpkg->installed.maintainer= 0;
-    otherpkg->installed.installedsize= otherpkg->installed.source= 0;
-    otherpkg->installed.origin= otherpkg->installed.bugs= 0;
-    otherpkg->installed.architecture= 0;
-    otherpkg->installed.conffiles= 0;
+    otherpkg->installed.description = otherpkg->installed.maintainer = NULL;
+    otherpkg->installed.installedsize = otherpkg->installed.source = NULL;
+    otherpkg->installed.origin = otherpkg->installed.bugs = NULL;
+    otherpkg->installed.architecture = NULL;
+    otherpkg->installed.conffiles = NULL;
     blankversion(&otherpkg->installed.version);
-    otherpkg->installed.arbs= 0;
+    otherpkg->installed.arbs = NULL;
     otherpkg->clientdata->fileslistvalid= 0;
 
     modstatdb_note(otherpkg);
@@ -1043,7 +1114,7 @@ void process_archive(const char *filename) {
               cfile->namenode->name, divpkg ? divpkg->name : "<local>");
       }        
     } else {
-      divpkg= 0;
+      divpkg = NULL;
       debug(dbg_eachfile, "process_archive looking for overwriting `%s'",
             cfile->namenode->name);
     }
